/*
 * add.c
 * This file provides the add functions required by gnozip
 */

#include "common.h"
#include "exit.h"
#include "tar.h"
#include "zip.h"
#include "gzip.h"
#include "bzip2.h"
#include "add.h"

GList *
gz_add_selection_to_list (GtkFileSelection *gfsel, gchar **adddir)
{
	GList *sel;
	GList *retlist = NULL;
	gchar *seldir;
	gchar *putdir = NULL;
	gchar *selstring;
	gchar *liststring;
	int selyes = FALSE;
	gchar *slash;

	seldir = g_strdup (gtk_file_selection_get_filename (gfsel));
	if (seldir[strlen (seldir) -1] == '/') seldir[strlen (seldir) -1] = '\0';
	slash = strrchr (seldir, '/');
	*(slash + 1) = '\0';
	DEBUG ("add_filesel:", seldir)
	
	if (!workdir) workdir = g_strdup_printf ("/");
	DEBUG ("add_basedir:", workdir)

	switch (workdir_type)
	{
	case ADD_FULL_BUTTON:
		putdir = g_strdup (seldir);
		*adddir = g_strdup ("/");
		break;
	case ADD_REL_BUTTON:
		putdir = g_malloc0 (1);
		*adddir = g_strdup (seldir);
		break;
	case ADD_THIS_BUTTON:
		if (!strncmp (seldir, workdir, strlen (workdir)))
		{
			putdir = g_strdup (&seldir[strlen (workdir)]);
			*adddir = g_strdup (workdir);
		}
		else
		{
			/*TODO calculate ../ sequences to the path */
			/* USING FULL PATH if files are not in subdirs of workdir */
			putdir = g_strdup (seldir);
			*adddir = g_strdup ("/");
		}
		break;
	default:
		ERROR (_("Internal Error"), _("Wrong path type"));
	}
	DEBUG ("add_putdir:", putdir)
	DEBUG ("add_adddir:", *adddir)

	if (GTK_CLIST (gfsel->file_list)->selection)
	{
		selyes = TRUE;
		sel = GTK_CLIST (gfsel->file_list)->selection;
		while (sel)
		{
			gtk_clist_get_text (GTK_CLIST(gfsel->file_list),
					    (gint) sel->data, 0, &selstring);
			liststring = g_strdup_printf ("%s%s", putdir, selstring);
			DEBUG ("add_tempfile:", liststring);
			retlist = g_list_append (retlist, liststring);
			sel = sel->next;
		}
	}
	if (GTK_CLIST (gfsel->dir_list)->selection)
	{
		selyes = TRUE;
		sel = GTK_CLIST (gfsel->dir_list)->selection;
		while (sel)
		{
			gtk_clist_get_text (GTK_CLIST(gfsel->dir_list),
					    (gint) sel->data, 0, &selstring);
			liststring = g_strdup_printf ("%s%s", putdir, selstring);
			DEBUG ("add_tempfile:", liststring);
			retlist = g_list_append (retlist, liststring);
			sel = sel->next;
		}
	}
	if (!selyes) DEBUG ("gz_add_selection_to_list:", "No files selected");
	
	g_free (putdir);
	g_free (seldir);
	return retlist;
}
	

int
gz_add (GList *add_list, gchar *add_dir)
{
  
	switch (filetype)
	{
	case FILE_ZIP:
		gz_zip_add (filename, add_list, add_dir, comp_lev);
		break;
	case FILE_GZIP:
		ERROR ("gz_add", _("Function not yet implemented"))
		break;
	case FILE_BZIP2:
		ERROR ("gz_add", _("Function not yet implemented"))
		break;
	case FILE_TGZ:
		gz_tar_add (filename, add_list, add_dir, TAR_GZ);
		break;
	case FILE_TBZ2:
		gz_tar_add (filename, add_list, add_dir, TAR_BZ2);
		break;
	case FILE_TAR:
		gz_tar_add (filename, add_list, add_dir, TAR_TAR);
		break;
	case FILE_ARJ:
		ERROR ("gz_add", _("Function not yet implemented"))
		break;
	case FILE_LHA:
		ERROR ("gz_add", _("Function not yet implemented"))
		break;
	case FILE_RAR:
		ERROR ("gz_add", _("Function not yet implemented"))
		break;
	default:
		ERROR ("gz_add", _("Internal Error"))
	}

	return 0;
}

