/*
 * zip.c
 * This file provides the zip file operations interface
 */
#include <stdio.h>
#include <sys/stat.h>
#include "common.h"
#include "bzip2.h"

extern gz_config config;

GList *
gz_bzip2_list (const gchar *filename)
{
  GList *list = NULL;
  char  *command;
  FILE  *input = NULL;
  gz_list_entry *entry;

  command = g_strdup_printf ("%s -t -v \"%s\"", config.progs.bunzip2, filename);
  DEBUG ("bunzip2 command:", command)

	  /* FIXME bzip2 output status messages always to stderr */
	  
  input = popen (command, "r");
  if (input == NULL) ERROR ("gz_bzip2_list", _("cannot execute bunzip2 command"))

  {
    char *name;
    char entrystr [255 +PATH_MAX +NAME_MAX];
    char *pes, *stemp; /* entry string pointer */
    int len; /* string lenght */

    /* FIXME input not retrieved because bzip2 write status on stderr */
    
    /* Read a line at a time */
    entrystr[0] = '\0';
/*    fgets (entrystr, 0 +0 +11 +9 +6 +4095 +255, input);
    if (ferror (input)) FATAL_ERROR ("gz_bzip2_list:",strerror (ferror (input))) */
        /* Check if the line is not too long */
/*    if (feof (input)) FATAL_ERROR ("gz_bzip2_list:", _("Error retrieving file list"))
    if (entrystr[strlen (entrystr) - 1] != '\n')
        FATAL_ERROR ("gz_bzip2_list:", _("Too many chars in entry"))
    else entrystr[strlen (entrystr) - 1] = '\0'; */
/*    DEBUG ("gz_bzip2_list:", entrystr) */
	    /* drop first line */
	    
    /* BZIP2 does not provide any kind of info on compressed file */
    /* this section will be used to test file integrity */

	    
    entry = g_malloc0 (sizeof(gz_list_entry));

    /* FIXME a standard "N/A" string for unavailable data ? */
    
    entry->perms = NULL; /* info not available */
    entry->owner = NULL; /* info not available */
    entry->date = NULL; /* info not available (may be set to file date in future) */
    entry->time = NULL; /* info not available */
    entry->size = NULL; /* info not available */

    /* FIXME find a better way to get filename (or ask?) */
    /* strip path */
    pes = strdup (filename);
    if ((stemp = strrchr(pes, '/')) == NULL) name = pes;
    else name = ++stemp;
    /* strip extension */
    if ((stemp = strrchr(name, '.')) != NULL) 
	if (!(strcmp (stemp, ".bz2"))) *stemp = '\0';
    
    if ((len = strlen (name)) > NAME_MAX) 
    {
      WARNING ("gz_bzip2_list:", _("name field too long"))
      len = NAME_MAX;
    }
    entry->name = g_malloc0 (len+1);
    strncpy (entry->name, name, len);
    DEBUG ("gz_bzip2_list:", entry->name); 

    list = g_list_append (list, entry);
    
  }

  pclose (input);

  return list;

}

int
gz_bzip2_extract (gchar *filename, gchar *extr_dir, gchar *name)
{
  char *command;
  char *errstr;
  char *outfile;
  FILE  *input = NULL;
  FILE *output = NULL;
  int writing = TRUE;
  gchar *barwname = "appbar1";
  gint outtotal = 0;
  gint innum, outnum;
  GtkWidget *mainappbar;
  struct stat fileprop;
  
  mainappbar = lookup_widget (main_win, barwname);

  if ( extr_dir[strlen (extr_dir) -1] == '/') extr_dir[strlen (extr_dir) -1] = '\0';
  outfile = g_strdup_printf ("%s/%s", extr_dir, name);
  DEBUG ("gz_bzip2_extract", outfile);
  
  if (stat (filename, &fileprop))
  {
	  ERROR ("gz_bzip2_extract", _("Unable to stat compressed file"))
	  perror ("gz_bzip2_extract");
	  return -1;
  }
  
  if ((input = fopen (filename, "r")) == NULL)
  {
	  ERROR ("gz_bzip2_extract", _("Unable to open compressed file"))
          perror ("gz_bzip2_extract");
          return -1;
  }

  errstr = g_strdup_printf ("Extract %s to: %s\n", filename, outfile);
  DEBUG ("gz_bzip2_extract:", errstr);
  g_free (errstr);
  command = g_strdup_printf ("%s > \"%s\"", config.progs.bunzip2, outfile);
  DEBUG ("bunzip2 command:", command);
  output = popen (command, "w");

  while (writing == TRUE)
  {
    char entrystr [8192];
    /* change with file got from stdout and update appbar at regular intervals */

    innum = fread (entrystr, 1, 8192, input);
    if (innum < 8192)
    {
      if (ferror (input)) FATAL_ERROR ("gz_bzip2_extract:",strerror (ferror (input)))
      if (feof (input)) writing = FALSE;
    }
    outnum = fwrite (entrystr, 1, innum, output);
    if (outnum < innum)
    {
      if (ferror (input)) FATAL_ERROR ("gz_bzip2_extract:",strerror (ferror (input)))
    }
    outtotal+=outnum;
    /* update prograss bar */
    gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) outtotal/fileprop.st_size);
    while (gtk_events_pending ())
                gtk_main_iteration ();
  }
  
  fclose (input);
  pclose (output);
  g_free (outfile);
  g_free (command);
		  
  return FALSE;
}
