/*
 * common.c contains shared data objects and common helpers functions.
 */

#include "common.h"
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

GtkWidget *add_win;
GtkWidget *extract_win;
GtkWidget *new_win;
GtkWidget *main_win;
GtkWidget *about_gnozip;
GtkWidget *error_dialog;
GtkWidget *pref_win;
GtkWidget *mime_dialog;
GtkWidget *quest_dialog;

GdkPixmap *dir_icon_pixmap;
GdkBitmap *dir_icon_mask;
GdkPixmap *link_icon_pixmap;
GdkBitmap *link_icon_mask;

gboolean fileopen = FALSE;
GList *filelist = NULL;
gchar *arg_filename = NULL;
gchar *filename = NULL;
gchar *savename = NULL;
gchar *workdir = NULL;
gchar *extract_dir = NULL;
gint workdir_type = ADD_REL_BUTTON;
gint comp_lev = 6;
gint flnum = 0;	/* number pf files */
gz_archive_type filetype = FILE_UNKN;
gboolean filesaved = TRUE;
gboolean overwrite = FALSE;
gboolean saveonexit = TRUE;
gz_config config;
gz_config tempcfg;

gz_archive_type new_archive_type = FILE_TGZ;
gint new_comp_lev = 6;
gchar *new_filename = NULL;

void
gz_mainappbar_set_status (gchar *status_string)
{
	static GtkWidget *mainappbar = NULL;
	const gchar *barwname = "appbar1";

	if (mainappbar == NULL)
	{
		DEBUG ("gz_mainappbar_set_status", "Retieving main clist")
		mainappbar = lookup_widget (main_win, barwname);
		if (mainappbar == NULL)
		{
			ERROR ("gz_mainappbar_set_status", _("Internal error!"))
			return;
		}
	}
	gnome_appbar_set_status (GNOME_APPBAR (mainappbar) ,status_string);
}

GtkWidget*
gz_mainclist_get_widget ()
{
	static GtkWidget *mainclist = NULL;
	const gchar *wname = "clist1";

	if (mainclist == NULL)
	{
		DEBUG ("gz_mainclist_get_widget", "Retieving main appbar")
		mainclist = lookup_widget (main_win, wname);
		if (mainclist == NULL)
		{
			ERROR ("gz_mainclist_get_widget", _("Internal error!"))
			return NULL;
		}
	}
	return mainclist;
}

/**
 * gz_clear_filelist:
 * @**list:	Pointer to Glist used to create the clist.
 * @*clist:	the clist widget to bee cleared.
 *
 * Helper function mainly intended to clean the main windows clist.
 * ATTENTION: It will g_list_free the list you pass so do not use this
 * function if you still need the list.
 *
 * returns nothing.
 */

void
gz_clear_filelist (GList **list, GtkCList *clist)
{
	GList *current = NULL;

	current = *list;

	gtk_clist_freeze (clist);
	gtk_clist_clear (clist);
	gtk_clist_thaw (clist);
    
	while (current)
	{
		gz_list_entry *data;

		data = current->data;
		g_free (data->perms);
		g_free (data->owner);
		g_free (data->size);
		g_free (data->date);
		g_free (data->time);
		g_free (data->name);
		g_free (current->data);
		current = current->next;
	}
	g_list_free (*list);
	*list = NULL;
}

GList*
gz_get_selected_filenames (GtkCList *clist)
{
	GList *row_list = clist->row_list;
	GList *filenames = NULL;
	gchar *filename;
	int i;
	
	i = 0;
	while (row_list)
	{
		if (GTK_CLIST_ROW (row_list)->state == GTK_STATE_SELECTED)
		{
			gz_list_entry *entry;
			gchar *name;
			gchar *path;

			gtk_clist_get_text (clist, i, COL_PATH, &path);
			gtk_clist_get_text (clist, i, COL_NAME, &name);
			filename = g_strdup_printf ("%s%s", path, name);
			filenames = g_list_append (filenames, filename);
			DEBUG("gz_get_selected_filenames", filename);
		}

		row_list = row_list->next;
		i++;
	}

	return filenames;
}

gint make_rdir (gchar *path)
{
	struct stat dir;
	gchar *pos;
	gchar *spath;
	gint ret;
	
	if (!path) return -1;
	if (!strlen (path)) return -1;
	
	spath = g_strdup (path);
	if (spath[strlen (spath) - 1] == '/') spath[strlen (spath) - 1] = '\0';
	pos = strrchr (spath, '/');
	if (!pos)
	{
		g_free (spath);
		return 1;
	}
	*pos = '\0';
	
	ret = stat (spath, &dir);
	if (ret)
	{
		if (errno == ENOENT)
		{
			ret = make_rdir (spath);
			if (ret) return ret;
		}
		else
		{
			g_free (spath);
			return 1;
		}
	}
	
	g_free (spath);	
	
	ret = mkdir (path, 0x1FF);
	if (ret) return 2;
	else return 0;
}
