/*
 * common.h contains common data types and helper functions declarations
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include "interface.h"
#include "support.h"

#ifdef SOLARIS
#include <dirent.h>
#define NAME_MAX MAXNAMLEN
#endif

#define TEXT_URI_LIST 1

#ifdef GZ_DEBUG
 #define DEBUG(head,message) g_print ("GnoZip: %s %s\n",head,message);
#else
 #define DEBUG(a,b)
#endif /* GZ_DEBUG */
#define ERROR(head,message) gtk_widget_show (create_error_dialog(message));
/* #define ERROR(head,message) gnome_app_error (GNOME_APP (main_win), message); */
#define WARNING(head,message) g_warning ("GnoZip: %s %s\n",head,message);
#define FATAL_ERROR(head,message) g_error ("GnoZip: %s %s\n",head,message);

#define TEMPFILE "gnozipXXXXXX"

#define XPM_DIR PACKAGE_PIXMAPS_DIR

#define S_ERR -1
#define S_OUT 0
#define S_DAT 1
#define S_RST 2

/* clists column default sizes (see gconfig.c) */
#define CDV_ICON FALSE
#define CDV_NAME TRUE
#define CDV_PERMS TRUE
#define CDV_OWNER FALSE
#define CDV_SIZE FALSE
#define CDV_DATE TRUE
#define CDV_TIME FALSE
#define CDV_PATH TRUE

#define DW_WIDTH 600
#define DW_HEIGHT 500

#define DS_ICON 16
#define DS_NAME 88
#define DS_PERMS 80
#define DS_OWNER 80
#define DS_SIZE 80
#define DS_DATE 72
#define DS_TIME 64
#define DS_PATH 128

#define ADD_FULL_BUTTON 1
#define ADD_REL_BUTTON 2
#define ADD_THIS_BUTTON 3

typedef enum   {
		COL_ICON,
		COL_NAME,
		COL_PERMS,
		COL_OWNER,
		COL_SIZE,
		COL_DATE,
		COL_TIME,
		COL_PATH
} gz_column_type;

typedef enum   {
		FILE_UNKN,
		FILE_ZIP,
		FILE_GZIP,
		FILE_BZIP2,
		FILE_TGZ,
		FILE_TBZ2,
		FILE_TAR,
		FILE_ARJ,
		FILE_LHA,
		FILE_RAR
} gz_archive_type;

typedef struct {
		gchar *zip;
		gchar *unzip;
		gchar *gzip;
		gchar *gunzip;
		gchar *bzip2;
		gchar *bunzip2;
		gchar *tar;
		gchar *arj;
		gchar *lha;
		gchar *rar;
		gchar *viewer;
} gz_progs;

typedef struct {
		gchar *defdir;
		gchar *scratch;
		gchar *xpm;
} gz_dirs;

typedef struct {
		gboolean icon;
		gboolean name;
		gboolean perms;
		gboolean owner;
		gboolean size;
		gboolean date;
		gboolean time;
		gboolean path;
} gz_view;

typedef struct {
		gint icon;
		gint name;
		gint perms;
		gint owner;
		gint size;
		gint date;
		gint time;
		gint path;
		gint main_width;
		gint main_height;
		/* gint main_x_pos;
		 * gint main_y_pos; */
} gz_dims;


typedef struct {
		gz_progs progs;
		gz_dirs  dirs;
		gz_view  view;
		gz_dims  dims;
} gz_config;	

typedef struct {
		gpointer *icon;
		gchar *name;
		gchar *perms;
		gchar *owner;
		gchar *size;
		gchar *date;
		gchar *time;
		gchar *path;
} gz_list_entry;

extern GtkWidget *add_win;
extern GtkWidget *extract_win;
extern GtkWidget *new_win;
extern GtkWidget *main_win;
extern GtkWidget *about_gnozip;
extern GtkWidget *error_dialog;
extern GtkWidget *pref_win;
extern GtkWidget *mime_dialog;
extern GtkWidget *quest_dialog;

extern GdkPixmap *dir_icon_pixmap;
extern GdkBitmap *dir_icon_mask;
extern GdkPixmap *link_icon_pixmap;
extern GdkBitmap *link_icon_mask;

extern gboolean fileopen;
extern GList *filelist;
extern gchar *filename;
extern gchar *arg_filename;
extern gchar *savename;
extern gchar *workdir;
extern gchar *extract_dir;
extern gint workdir_type;
extern gint comp_lev;
extern gint flnum;
extern gz_archive_type filetype;
extern gboolean filesaved;
extern gboolean overwrite;
extern gboolean saveonexit;
extern gz_config config;
extern gz_config tempcfg;

extern gz_archive_type new_archive_type;
extern gint new_comp_lev;
extern gchar *new_filename;

void gz_mainappbar_set_status (gchar *status_string);
GtkWidget* gz_mainclist_get_widget (void);
void gz_clear_filelist (GList **list, GtkCList *clist);
GList* gz_get_selected_filenames (GtkCList *clist);
gint make_rdir (gchar *path);
