/*
 * extract.c
 * This file provides the extract functions required by gnozip
 */

#include "common.h"
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include "exit.h"
#include "tar.h"
#include "zip.h"
#include "gzip.h"
#include "bzip2.h"
#include "extract.h"

int
gz_extract ()
{
  GtkWidget *main_clist = NULL;
  gchar *name;
  gz_list_entry *data;
  gint filelenght = 0;
  GList *extract_file_list;
  struct stat extdr;
  gint ret;
  
  main_clist = gz_mainclist_get_widget ();
  extract_file_list = gz_get_selected_filenames (GTK_CLIST(main_clist));
  
  if (ret = stat (extract_dir, &extdr))
  {
      DEBUG("gz_extract", "directory does not exist\n");
	
      if (errno == ENOENT)
      {
          ret = -1;
          quest_dialog = create_quest_dialog (	_("No directory"),
						_("The directory does not exist!\nDo you want to create it?"),
						&ret);
	  gtk_widget_show (quest_dialog);
          while (ret == -1)
	  {
	  	while (gtk_events_pending ()) gtk_main_iteration ();
		usleep (10000);
	}
	  if (ret) return 1;
          else
          {
	      ret = make_rdir (extract_dir);
	      if (ret)
	      {
		  ERROR ("gz_extract:", _("Unable to create directory!"))
		  return -1;
              }
          }
      }
      else
      {
          ERROR ("gz_extract:", _("Invalid directory, or pathname too long!"))
          return -1;
      }
  }
  
  switch (filetype)
  {
   case FILE_ZIP:
     gz_zip_extract (filename, extract_dir);
     break;
   case FILE_GZIP:
     data = filelist->data;
     name = data->name;
     filelenght = atoi (data->size);
     gz_gzip_extract (filename, filelenght, extract_dir, name);
     break;
   case FILE_BZIP2:
     data = filelist->data;
     name = data->name;
     gz_bzip2_extract (filename, extract_dir, name);
     break;
   case FILE_TGZ:
     gz_tar_extract (filename, extract_file_list, extract_dir, TAR_GZ);
     break;
   case FILE_TBZ2:
     gz_tar_extract (filename, extract_file_list, extract_dir, TAR_BZ2);
     break;
   case FILE_TAR:
     gz_tar_extract (filename, extract_file_list, extract_dir, TAR_TAR);
     break;
   case FILE_ARJ:
     ERROR ("gz_extract:", _("Function not yet implemented"))
     break;
   case FILE_LHA:
     ERROR ("gz_extract:", _("Function not yet implemented"))
     break;
   case FILE_RAR:
     ERROR ("gz_extract:", _("Function not yet implemented"))
     break;
   default:
     ERROR ("gz_extract:", _("no archives open"))
  }

  /*FIXME: free the Glist and it's contents */

  return 0;
}

