/*
 * config.c provides functions to manipulate configuration files
 */

#include "common.h"
#include "gconfig.h"

/** gz_config_init:
 *
 * To be called at program initialization.
 * Read and check config files.
 * If config files are missing, create new ones with default values.
 *
 * return nothing.
 */

void
gz_config_init () {
	
  gboolean def;

  /* custom view */
  config.view.icon = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/icon=%s" ,(CDV_ICON?"TRUE":"FALSE")), &def);
  if (def) gnome_config_private_set_bool ("gnozip/view/icon", config.view.icon);
  
  config.view.name = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/name=%s", (CDV_NAME?"TRUE":"FALSE")), &def);
  if (def) gnome_config_private_set_bool ("gnozip/view/name", config.view.name);
  
  config.view.perms = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/perms=%s", (CDV_PERMS?"TRUE":"FALSE")), &def);
  if (def) gnome_config_private_set_bool ("gnozip/view/perms", config.view.perms);
  
  config.view.owner = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/owner=%s", (CDV_OWNER?"TRUE":"FALSE")), &def);
  if (def) gnome_config_private_set_bool ("gnozip/view/owner", config.view.owner);

  config.view.size = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/size=%s", (CDV_SIZE?"TRUE":"FALSE")), &def);
  if (def) gnome_config_private_set_bool ("gnozip/view/size", config.view.size);

  config.view.date = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/date=%s", (CDV_DATE?"TRUE":"FALSE")), &def);
  if (def) gnome_config_private_set_bool ("gnozip/view/date", config.view.date);

  config.view.time = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/time=%s", (CDV_TIME?"TRUE":"FALSE")), &def);
  if (def) gnome_config_private_set_bool ("gnozip/view/time", config.view.time);
  
  config.view.path = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/path=%s", (CDV_PATH?"TRUE":"FALSE")), &def);
  if (def) gnome_config_private_set_bool ("gnozip/view/path", config.view.path);

  /* custom dims */
  config.dims.main_width = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/main_width=%d", DW_WIDTH), &def);
  if (def) gnome_config_private_set_int ("gnozip/dims/main_width", config.dims.main_width);
  config.dims.main_height = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/main_height=%d", DW_HEIGHT), &def);
  if (def) gnome_config_private_set_int ("gnozip/dims/main_height", config.dims.main_height);

  config.dims.icon = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/icon=%d", DS_ICON), &def);
  if (def) gnome_config_private_set_int ("gnozip/dims/icon", config.dims.icon);
  config.dims.name = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/name=%d", DS_NAME), &def);
  if (def) gnome_config_private_set_int ("gnozip/dims/name", config.dims.name);
  config.dims.perms = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/perms=%d", DS_PERMS), &def);
  if (def) gnome_config_private_set_int ("gnozip/dims/perms", config.dims.perms);
  config.dims.owner = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/owner=%d", DS_OWNER), &def);
  if (def) gnome_config_private_set_int ("gnozip/dims/owner", config.dims.owner);
  config.dims.size = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/size=%d", DS_SIZE), &def);
  if (def) gnome_config_private_set_int ("gnozip/dims/size", config.dims.size);
  config.dims.date = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/date=%d", DS_DATE), &def);
  if (def) gnome_config_private_set_int ("gnozip/dims/date", config.dims.date);
  config.dims.time = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/time=%d", DS_TIME), &def);
  if (def) gnome_config_private_set_int ("gnozip/dims/time", config.dims.time);
  config.dims.path = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/path=%d", DS_PATH), &def);
  if (def) gnome_config_private_set_int ("gnozip/dims/path", config.dims.path);
  
  /* directories */
  config.dirs.defdir = gnome_config_private_get_string_with_default ( g_strdup_printf ("gnozip/dirs/defdir=%s", g_get_home_dir ()), &def);
  if (def) gnome_config_private_set_string ("gnozip/dirs/defdir", config.dirs.defdir);
  
  config.dirs.scratch = gnome_config_private_get_string_with_default (g_strdup_printf ("gnozip/dirs/scratch=%s", g_get_tmp_dir ()), &def);
  if (def) gnome_config_private_set_string ("gnozip/dirs/scratch", config.dirs.scratch);

  config.dirs.xpm = gnome_config_private_get_string_with_default ( g_strdup_printf ("gnozip/dirs/xpm=%s", XPM_DIR), &def);
  if (def) gnome_config_private_set_string ("gnozip/dirs/xpm", config.dirs.xpm);

  /*executables */
  config.progs.zip = gnome_config_private_get_string_with_default ("gnozip/progs/zip=/usr/bin/zip", &def);
  if (def) gnome_config_private_set_string ("gnozip/progs/zip", config.progs.zip);
  config.progs.unzip = gnome_config_private_get_string_with_default ("gnozip/progs/unzip=/usr/bin/unzip", &def);
  if (def) gnome_config_private_set_string ("gnozip/progs/unzip", config.progs.unzip);
  config.progs.tar = gnome_config_private_get_string_with_default ("gnozip/progs/tar=/bin/tar", &def);
  if (def) gnome_config_private_set_string ("gnozip/progs/tar", config.progs.tar);
  config.progs.gzip = gnome_config_private_get_string_with_default ("gnozip/progs/gzip=/usr/bin/gzip", &def);
  if (def) gnome_config_private_set_string ("gnozip/progs/gzip", config.progs.gzip);
  config.progs.gunzip = gnome_config_private_get_string_with_default ("gnozip/progs/gunzip=/usr/bin/gunzip", &def);
  if (def) gnome_config_private_set_string ("gnozip/progs/gunzip", config.progs.gunzip);
  config.progs.bzip2 = gnome_config_private_get_string_with_default ("gnozip/progs/bzip2=/usr/bin/bzip2", &def);
  if (def) gnome_config_private_set_string ("gnozip/progs/bzip2", config.progs.bzip2);
  config.progs.bunzip2 = gnome_config_private_get_string_with_default ("gnozip/progs/bunzip2=/usr/bin/bunzip2", &def);
  if (def) gnome_config_private_set_string ("gnozip/progs/bunzip2", config.progs.bunzip2);
  config.progs.arj = gnome_config_private_get_string_with_default ("gnozip/progs/arj=/usr/bin/arj", &def);
  if (def) gnome_config_private_set_string ("gnozip/progs/arj", config.progs.arj);
  config.progs.lha = gnome_config_private_get_string_with_default ("gnozip/progs/lha=/usr/bin/lha", &def);
  if (def) gnome_config_private_set_string ("gnozip/progs/lha", config.progs.lha);
  config.progs.rar = gnome_config_private_get_string_with_default ("gnozip/progs/rar=/usr/bin/rar", &def);
  if (def) gnome_config_private_set_string ("gnozip/progs/rar", config.progs.rar);
  config.progs.viewer = gnome_config_private_get_string_with_default ("gnozip/progs/viewer=/usr/bin/gnp", &def);
  if (def) gnome_config_private_set_string ("gnozip/progs/viewer", config.progs.viewer);

  gnome_config_sync ();
}

/** gz_config_read:
 *
 * Read config files.
 * If config files are missing, sets default values.
 *
 * return nothing.
 */

void
gz_config_read () {
	
  gboolean def;

  /* custom view */
  config.view.icon = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/icon=%s", (CDV_ICON?"TRUE":"FALSE")), &def);
  config.view.name = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/name=%s", (CDV_NAME?"TRUE":"FALSE")), &def);
  config.view.perms = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/perms=%s", (CDV_PERMS?"TRUE":"FALSE")), &def);
  config.view.owner = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/owner=%s", (CDV_OWNER?"TRUE":"FALSE")), &def);
  config.view.size = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/size=%s", (CDV_SIZE?"TRUE":"FALSE")), &def);
  config.view.date = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/date=%s", (CDV_DATE?"TRUE":"FALSE")), &def);
  config.view.time = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/time=%s", (CDV_TIME?"TRUE":"FALSE")), &def);
  config.view.path = gnome_config_private_get_bool_with_default (g_strdup_printf ("gnozip/view/path=%s", (CDV_PATH?"TRUE":"FALSE")), &def);

  /* custom dims */
  config.dims.main_width = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/main_width=%d", DW_WIDTH), &def);
  config.dims.main_height = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/main_height=%d", DW_HEIGHT), &def);

  config.dims.icon = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/icon=%d", DS_ICON), &def);
  config.dims.name = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/name=%d", DS_NAME), &def);
  config.dims.perms = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/perms=%d", DS_PERMS), &def);
  config.dims.owner = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/owner=%d", DS_OWNER), &def);
  config.dims.size = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/size=%d", DS_SIZE), &def);
  config.dims.date = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/date=%d", DS_DATE), &def);
  config.dims.time = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/time=%d", DS_TIME), &def);
   config.dims.path = gnome_config_private_get_int_with_default (g_strdup_printf ("gnozip/dims/path=%d", DS_PATH), &def);


  /* directories */
  config.dirs.defdir = gnome_config_private_get_string_with_default (g_strdup_printf ("gnozip/dirs/defdir=%s", g_get_home_dir ()), &def);

  config.dirs.scratch = gnome_config_private_get_string_with_default (g_strdup_printf ("gnozip/dirs/scratch=%s", g_get_tmp_dir ()), &def);

  config.dirs.xpm = gnome_config_private_get_string_with_default (g_strdup_printf ("gnozip/dirs/xpm=%s", XPM_DIR), &def);

  /*executables */
  g_free (config.progs.zip);
  config.progs.zip = gnome_config_private_get_string_with_default ("gnozip/progs/zip=/usr/bin/zip", &def);
  g_free (config.progs.unzip);
  config.progs.unzip = gnome_config_private_get_string_with_default ("gnozip/progs/unzip=/usr/bin/unzip", &def);
  g_free (config.progs.tar);
  config.progs.tar = gnome_config_private_get_string_with_default ("gnozip/progs/tar=/bin/tar", &def);
  g_free (config.progs.gzip);
  config.progs.gzip = gnome_config_private_get_string_with_default ("gnozip/progs/gzip=/usr/bin/gzip", &def);
  g_free (config.progs.gunzip);
  config.progs.gunzip = gnome_config_private_get_string_with_default ("gnozip/progs/gunzip=/usr/bin/gunzip", &def);
  g_free (config.progs.bzip2);
  config.progs.bzip2 = gnome_config_private_get_string_with_default ("gnozip/progs/bzip2=/usr/bin/bzip2", &def);
  g_free (config.progs.bunzip2);
  config.progs.bunzip2 = gnome_config_private_get_string_with_default ("gnozip/progs/bunzip2=/usr/bin/bunzip2", &def);
  g_free (config.progs.arj);
  config.progs.arj = gnome_config_private_get_string_with_default ("gnozip/progs/arj=/usr/bin/arj", &def);
  g_free (config.progs.lha);
  config.progs.lha = gnome_config_private_get_string_with_default ("gnozip/progs/lha=/usr/bin/lha", &def);
  g_free (config.progs.rar);
  config.progs.rar = gnome_config_private_get_string_with_default ("gnozip/progs/rar=/usr/bin/rar", &def);
  g_free (config.progs.viewer);
  config.progs.viewer = gnome_config_private_get_string_with_default ("gnozip/progs/viewer=/usr/bin/gnp", &def);
}

/** gz_config_default:
 *
 * Set default values.
 *
 * return nothing.
 */

void
gz_config_default () {
	
  /* custom view */
  config.view.icon = CDV_ICON;
  config.view.name = CDV_NAME;
  config.view.perms = CDV_PERMS;
  config.view.owner = CDV_OWNER;
  config.view.size = CDV_SIZE;
  config.view.date = CDV_DATE;
  config.view.time = CDV_TIME;
  config.view.path = CDV_PATH;

  /* custom dims */
  config.dims.main_width = DW_WIDTH;
  config.dims.main_height = DW_HEIGHT;

  config.dims.icon = DS_ICON;
  config.dims.name = DS_NAME;
  config.dims.perms = DS_PERMS;
  config.dims.owner = DS_OWNER;
  config.dims.size = DS_SIZE;
  config.dims.date = DS_DATE;
  config.dims.time = DS_TIME;
  config.dims.path = DS_PATH;
  
  /* directories */
  g_free (config.dirs.defdir);
  config.dirs.defdir = g_get_home_dir ();
  g_free (config.dirs.scratch);
  config.dirs.scratch = g_get_tmp_dir ();
  g_free (config.dirs.xpm);
  config.dirs.xpm = XPM_DIR;

  /*executables */
  g_free (config.progs.zip);
  config.progs.zip = g_strdup ("gnozip/progs/zip");
  g_free (config.progs.unzip);
  config.progs.unzip = g_strdup ("/usr/bin/unzip");
  g_free (config.progs.tar);
  config.progs.tar = g_strdup ("/bin/tar");
  g_free (config.progs.gzip);
  config.progs.gzip = g_strdup ("/usr/bin/gzip");
  g_free (config.progs.gunzip);
  config.progs.gunzip = g_strdup ("/usr/bin/gunzip");
  g_free (config.progs.bzip2);
  config.progs.bzip2 = g_strdup ("/usr/bin/bzip2");
  g_free (config.progs.bunzip2);
  config.progs.bunzip2 = g_strdup ("/usr/bin/bunzip2");
  g_free (config.progs.arj);
  config.progs.arj = g_strdup ("/usr/bin/arj");
  g_free (config.progs.lha);
  config.progs.lha = g_strdup ("/usr/bin/lha");
  g_free (config.progs.rar);
  config.progs.rar = g_strdup ("/usr/bin/rar");
  g_free (config.progs.viewer);
  config.progs.viewer = g_strdup ("/usr/bin/gnp");
}

/** gz_config_write:
 *
 * Write current values to config files.
 *
 * return nothing.
 */

void
gz_config_write () {

  DEBUG ("gz_config","write configuration")
	
  /* custom view */
  gnome_config_private_set_bool ("gnozip/view/icon", config.view.icon);
  gnome_config_private_set_bool ("gnozip/view/name", config.view.name);
  gnome_config_private_set_bool ("gnozip/view/perms", config.view.perms);
  gnome_config_private_set_bool ("gnozip/view/owner", config.view.owner);
  gnome_config_private_set_bool ("gnozip/view/size", config.view.size);
  gnome_config_private_set_bool ("gnozip/view/date", config.view.date);
  gnome_config_private_set_bool ("gnozip/view/time", config.view.time);
  gnome_config_private_set_bool ("gnozip/view/path", config.view.path);

  /* custom dims */
  gnome_config_private_set_int ("gnozip/dims/main_width", config.dims.main_width);
  gnome_config_private_set_int ("gnozip/dims/main_height", config.dims.main_height);

  gnome_config_private_set_int ("gnozip/dims/icon", config.dims.icon);
  gnome_config_private_set_int ("gnozip/dims/name", config.dims.name);
  gnome_config_private_set_int ("gnozip/dims/perms", config.dims.perms);
  gnome_config_private_set_int ("gnozip/dims/owner", config.dims.owner);
  gnome_config_private_set_int ("gnozip/dims/size", config.dims.size);
  gnome_config_private_set_int ("gnozip/dims/date", config.dims.date);
  gnome_config_private_set_int ("gnozip/dims/time", config.dims.time);
  gnome_config_private_set_int ("gnozip/dims/path", config.dims.path);

  /* directories */
  gnome_config_private_set_string ("gnozip/dirs/defdir", config.dirs.defdir);
  gnome_config_private_set_string ("gnozip/dirs/scratch", config.dirs.scratch);
  gnome_config_private_set_string ("gnozip/dirs/xpm", config.dirs.xpm);

  /*executables */
  gnome_config_private_set_string ("gnozip/progs/zip", config.progs.zip);
  gnome_config_private_set_string ("gnozip/progs/unzip", config.progs.unzip);
  gnome_config_private_set_string ("gnozip/progs/tar", config.progs.tar);
  gnome_config_private_set_string ("gnozip/progs/gzip", config.progs.gzip);
  gnome_config_private_set_string ("gnozip/progs/gunzip", config.progs.gunzip);
  gnome_config_private_set_string ("gnozip/progs/bzip2", config.progs.bzip2);
  gnome_config_private_set_string ("gnozip/progs/bunzip2", config.progs.bunzip2);
  gnome_config_private_set_string ("gnozip/progs/arj", config.progs.arj);
  gnome_config_private_set_string ("gnozip/progs/lha", config.progs.lha);
  gnome_config_private_set_string ("gnozip/progs/rar", config.progs.rar);
  gnome_config_private_set_string ("gnozip/progs/gnp", config.progs.viewer);
  
  gnome_config_sync ();
}

void
gz_config_free (gz_config *cfg) {

	gz_progs *progs;
	gz_dirs  *dirs;

	dirs = &(cfg->dirs);
	g_free (dirs->defdir);
	dirs->defdir = NULL;
	g_free (dirs->scratch);
	dirs->scratch = NULL;
	g_free (dirs->xpm);
	dirs->xpm = NULL;

	progs = &(cfg->progs);
	g_free (progs->zip);
	progs->zip = NULL;
	g_free (progs->unzip);
	progs->unzip = NULL;
	g_free (progs->tar);
	progs->tar = NULL;
	g_free (progs->gzip);
	progs->gzip = NULL;
	g_free (progs->gunzip);
	progs->gunzip = NULL;
	g_free (progs->bzip2);
	progs->bzip2 = NULL;
	g_free (progs->bunzip2);
	progs->bunzip2 = NULL;
	g_free (progs->arj);
	progs->arj = NULL;
	g_free (progs->lha);
	progs->lha = NULL;
	g_free (progs->rar);
	progs->rar = NULL;
	g_free (progs->viewer);
	progs->viewer = NULL;
	
}

void
gz_config_cpy (gz_config *dest, gz_config *src) {


	gz_config_free (dest);
	
	dest->view.icon = src->view.icon;
	dest->view.name = src->view.name;
	dest->view.perms = src->view.perms;
	dest->view.owner = src->view.owner;
	dest->view.size = src->view.size;
	dest->view.date = src->view.date;
	dest->view.time = src->view.time;
	dest->view.path = src->view.path;
	
	dest->dims.icon = src->dims.icon;
	dest->dims.name = src->dims.name;
	dest->dims.perms = src->dims.perms;
	dest->dims.owner = src->dims.owner;
	dest->dims.size = src->dims.size;
	dest->dims.date = src->dims.date;
	dest->dims.time = src->dims.time;
	dest->dims.path = src->dims.path;
	dest->dims.main_width = src->dims.main_width;
	dest->dims.main_height = src->dims.main_height;
	
	if ((src->dirs.defdir))
		dest->dirs.defdir = g_strdup (src->dirs.defdir);
	if ((src->dirs.scratch))
		dest->dirs.scratch = g_strdup (src->dirs.scratch);
	if ((src->dirs.xpm))
		dest->dirs.xpm = g_strdup (src->dirs.xpm);

	if ((src->progs.zip))
		dest->progs.zip = g_strdup (src->progs.zip);
	if ((src->progs.unzip))
		dest->progs.unzip = g_strdup (src->progs.unzip);
	if ((src->progs.gzip))
		dest->progs.gzip = g_strdup (src->progs.gzip);
	if ((src->progs.gunzip))
		dest->progs.gunzip = g_strdup (src->progs.gunzip);
	if ((src->progs.bzip2))
		dest->progs.bzip2 = g_strdup (src->progs.bzip2);
	if ((src->progs.bunzip2))
		dest->progs.bunzip2 = g_strdup (src->progs.bunzip2);
	if ((src->progs.tar))
		dest->progs.tar = g_strdup (src->progs.tar);
	if ((src->progs.arj))
		dest->progs.arj = g_strdup (src->progs.arj);
	if ((src->progs.lha))
		dest->progs.lha = g_strdup (src->progs.lha);
	if ((src->progs.rar))
		dest->progs.rar = g_strdup (src->progs.rar);
	if ((src->progs.viewer))
		dest->progs.viewer = g_strdup (src->progs.viewer);

}

void
gz_set_prefs ()
{
	
	GtkWidget *widget;

	gz_config_cpy (&config, &tempcfg);

	widget = lookup_widget (main_win, "clist1");
	if (config.view.icon)	
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  0,
						  1);
	else	
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  0,
						  0);
	
	if (config.view.name)	
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  1,
						  1);
	else	
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  1,
						  0);
	
	if (config.view.perms)
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  2,
						  1);
	else	
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  2,
						  0);

	if (config.view.owner)
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  3,
						  1);
	else	
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  3,
						  0);

	if (config.view.size)
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  4,
						  1);
	else	
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  4,
						  0);

	if (config.view.date)
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  5,
						  1);
	else	
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  5,
						  0);

	if (config.view.time)
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  6,
						  1);
	else	
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  6,
						  0);

	if (config.view.path)
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  7,
						  1);
	else	
		gtk_clist_set_column_visibility ( GTK_CLIST (widget),
						  7,
						  0);

}

void
gz_icons_init ()
{
	GtkWidget *icon;
	gchar *xpmname;
	
	xpmname = g_strdup_printf ("%s/gnozip/directory.xpm", config.dirs.xpm);
	icon = create_pixmap (gz_mainclist_get_widget (), xpmname, 0);
	gtk_pixmap_get (GTK_PIXMAP (icon), &dir_icon_pixmap, &dir_icon_mask);

	xpmname = g_strdup_printf ("%s/gnozip/link.xpm", config.dirs.xpm);
	icon = create_pixmap (gz_mainclist_get_widget (), xpmname, 0);
	gtk_pixmap_get (GTK_PIXMAP (icon), &link_icon_pixmap, &link_icon_mask);
}
