/*
 * zip.c
 * This file provides the zip file operations interface
 */

#include <stdio.h>
#include "common.h"
#include "gzip.h"

extern gz_config config;

GList *
gz_gzip_list (const gchar *filename)
{
  GList *filelist = NULL;
  char  *command;
  FILE  *input = NULL;
  gz_list_entry *entry;

  command = g_strdup_printf ("%s -l -N -v \"%s\"", config.progs.gunzip, filename);
  DEBUG ("gzip command:", command)
  input = popen (command, "r");
  if (input == NULL) ERROR ("gz_gzip_list", _("cannot execute command"))

  {
    char *size, *datem, *dated, *time, *name; /*, *path;*/
    char entrystr [255 +PATH_MAX +NAME_MAX];
    char *pes; /* entry string pointer */
    char *stemp;
    int len; /* string lenght */

    /* Read a line at a time */
    entrystr[0] = '\0';
    fgets (entrystr, 255 +PATH_MAX +NAME_MAX, input);
    if (ferror (input)) FATAL_ERROR ("gz_gzip_list:",strerror (ferror (input)))
        /* Check if the line is not too long */
    if (feof (input)) FATAL_ERROR ("gz_gzip_list:", _("Error retrieving file list"))
    if (entrystr[strlen (entrystr) - 1] != '\n')
        FATAL_ERROR ("gz_gzip_list:", _("Too many chars in entry"))
    else entrystr[strlen (entrystr) - 1] = '\0';
    DEBUG ("gz_gzip_list:", entrystr)
	    /* drop first line */
	    
    fgets (entrystr, 255 +PATH_MAX +NAME_MAX, input);
    if (ferror (input)) FATAL_ERROR ("gz_gzip_list:",strerror (ferror (input)))
        /* Check if the line is not too long */
    if (feof (input)) FATAL_ERROR ("gz_gzip_list:", _("Error retrieving file list"))
    if (entrystr[strlen (entrystr) - 1] != '\n')
        FATAL_ERROR ("gz_gzip_list:", _("Too many chars in file entry"))
    else entrystr[strlen (entrystr) - 1] = '\0';
    DEBUG ("gz_gzip_list:", entrystr)

    entry = g_malloc0 (sizeof(gz_list_entry));

    entry->perms = NULL; /* info not available */
    entry->owner = NULL; /* info not available */
    
    pes = entrystr;

    g_strchug (pes); /* Get Out spaces */
    strsep(&pes, " "); /* just ignore method */
    g_strchug (pes); /* Get Out spaces */
    strsep(&pes, " "); /* just ignore crc */
    g_strchug (pes); /* Get Out spaces */

    datem = strsep(&pes, " ");
    g_strchug (pes); /* Get Out spaces */
    dated = strsep(&pes, " ");
    
    if ((len = (strlen (datem)) + (strlen (dated)) + 1) > 6)
    {
      WARNING ("gz_gzip_list:", _("date field too long"))
      len = 6;
    }
    entry->date = g_malloc0 (len+1);
    strncpy (entry->date, dated, strlen (dated));
    entry->date[strlen (dated)] = ' ';
    strncpy (entry->date + strlen (dated) +1, datem, strlen (datem));
/*    DEBUG ("gz_gzip_list:", entry->date) */
    
    g_strchug (pes); /* Get Out spaces */
    time = strsep(&pes, " ");
    if ((len = strlen (time)) > 5)
    {
      WARNING ("gz_zip_list:", _("time field too long"))
      len = 5;
    }
    entry->time = g_malloc0 (len+1);
    strncpy (entry->time, time, len);
/*    DEBUG ("gz_gzip_list:", entry->time); */

    g_strchug (pes); /* Get Out spaces */
    strsep(&pes, " "); /* just ignore  compressed size */
    g_strchug (pes); /* Get Out spaces */
    
    size = strsep(&pes, " ");
    if ((len = strlen (size)) > 10)
    {
      WARNING ("gz_gzip_list:", _("size field too long"))
      len = 10;
    }
    entry->size = g_malloc0 (len+1);
    strncpy (entry->size, size, len);
/*    DEBUG ("gz_gzip_list:", entry->size) */

    g_strchug (pes); /* Get Out spaces */
    strsep(&pes, " "); /* just ignore ratio */

    /* FIXME: may be name is stored with path in a gzipped file? */
    g_strchug (pes); /* Get Out spaces */
    if ((stemp = strrchr(pes, '/')) == NULL) name = pes;
    else name = ++stemp;
    if ((len = strlen (name)) > NAME_MAX) 
    {
      WARNING ("gz_gzip_list:", _("name field too long"))
      len = NAME_MAX;
    }
    entry->name = g_malloc0 (len+1);
    strncpy (entry->name, name, len);
/*    DEBUG ("gz_gzip_list:", entry->name); */

    filelist = g_list_append (filelist, entry);
    
  }

  pclose (input);
  g_free (command);

  return filelist;

}

int
gz_gzip_extract (gchar *filename, gint filelen, gchar *extr_dir, gchar *name)
{
  char *command;
  char *errstr;
  char *outfile;
  FILE  *input = NULL;
  FILE *output = NULL;
  int reading = TRUE;
  gchar *barwname = "appbar1";
  gint outtotal = 0;
  gint innum, outnum;
  GtkWidget *mainappbar;
  
  mainappbar = lookup_widget (main_win, barwname);

  if ( extr_dir[strlen (extr_dir) -1] == '/') extr_dir[strlen (extr_dir) -1] = '\0';
  outfile = g_strdup_printf ("%s/%s", extr_dir, name);
  DEBUG ("gz_gzip_extract", outfile);

  if ((output = fopen (outfile, "wb")) == NULL)
  {
	  ERROR ("gz_gzip_extract", _("Unable to open destination file"))
          perror ("gz_gzip_extract");
          return -1;
  }

  errstr = g_strdup_printf ("Extract %s to: %s/%s\n", filename, extr_dir, name);
  DEBUG ("gz_gzip_extract:", errstr);
  g_free (errstr);
  command = g_strdup_printf ("%s -v -c \"%s\"", config.progs.gunzip, filename);
  DEBUG ("gzip command:", command);

  input = popen (command, "r");
  while (reading == TRUE)
  {
    char entrystr [8192];
    /* change with file got from stdout and update appbar at regular intervals */

    innum = fread (entrystr, 1, 8192, input);
    if (innum < 8192)
    {
      if (ferror (input)) FATAL_ERROR ("gz_gzip_extract:",strerror (ferror (input)))
      if (feof (input)) reading = FALSE;
    }
    outnum = fwrite (entrystr, 1, innum, output);
    if (outnum < innum)
    {
      if (ferror (input)) FATAL_ERROR ("gz_gzip_extract:",strerror (ferror (input)))
    }
    outtotal+=outnum;
    /* update prograss bar */
    gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) outtotal/filelen);
    while (gtk_events_pending ())
                gtk_main_iteration ();
  }
  
  fclose (output);
  pclose (input);
  g_free (command);
  g_free (outfile);
  
  return FALSE;
}
