/*
 * tar.c
 * This file provides the tar, tar.gz, tar.bz2 file operations interface
 */

#include "common.h"
#include "tar.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

extern gz_config config;

GList *
gz_tar_list (const gchar *filename, int compression)
{
	GList *list = NULL;
	char  *command;
	FILE  *input = NULL;
	gz_list_entry *entry;
	char topt [5];
	gchar *barwname = "appbar1";
	/* gint innum = 0; */
	GtkWidget *mainappbar;
  
	mainappbar = lookup_widget (main_win, barwname);
	gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,0);
 
	switch (compression)
	{
	case 0:
		sprintf (topt, "tvf");
		break;
	case 1:
		sprintf (topt, "tvzf");
		break;
	case 2:
		sprintf (topt, "tvIf");
		break;
	default:
		FATAL_ERROR ("gz_tar_list:",_("Wrong Parameter"))
	}
	command = g_strdup_printf ("%s %s \"%s\"", config.progs.tar, topt, filename);
	DEBUG ("tar command:", command)
	input = popen (command, "r");
	if (input == NULL) ERROR ("gz_tar_list", _("cannot execute tar command"))

	/*
	 * Redirezionare stderror in modo da riportare gli errori del tar ? 
	 * E' necessario poi rimetterlo a posto ? 
	*/
  
	while (TRUE)
	{
		char entrystr [11 +18 +11 +11 +9 +PATH_MAX +NAME_MAX +1];
    
		/* Read a line at a time */
		entrystr[0] = '\0';
		fgets (entrystr, 11 +18 +11 +11 +9 +PATH_MAX +NAME_MAX, input);
		if (ferror (input)) FATAL_ERROR ("gz_tar_list:",strerror (ferror (input)))
		/* Check if the line is not too long */
		if (feof (input)) break;
		if (entrystr[strlen (entrystr) - 1] != '\n')
			FATAL_ERROR ("gz_tar_list:", _("Too many chars in a file entry"))
		else entrystr[strlen (entrystr) - 1] = '\0';
		DEBUG ("gz_tar_list:", entrystr)

		g_strstrip (entrystr);
		entry = gz_tar_entry (entrystr);
		list = g_list_append (list, entry);

	}
	/* gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) ++innum/????);
	 */
	
	gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,1);
	pclose (input);
	return list;
}

gz_list_entry *
gz_tar_entry (char * entrystr)
{
	gz_list_entry *entry;
	char *perms, *owner, *size, *date, *time, *name, *path;
	char *pes; /* entry string pointer */
	int len; /* string lenght */
	char lastslash = '\0';

	entry = g_malloc0 (sizeof (gz_list_entry));
   
	pes = strdup (entrystr);
	perms = strsep(&pes, " ");
	if ((len = strlen (perms)) > 10)
		WARNING ("gz_tar_list:", _("perms field too long"))
	entry->perms = g_strdup (perms);
	/*    DEBUG ("gz_tar_list:", entry->perms); */

	g_strchug (pes); /* Get Out spaces */
	owner = strsep (&pes, " ");
	if ((len = strlen (owner)) > 17)
		WARNING ("gz_tar_list:", _("owner field too long"))
	entry->owner = g_strdup (owner);
	/*    DEBUG ("gz_tar_list:", entry->owner); */
    
	g_strchug (pes); /* Get Out spaces */
	size = strsep(&pes, " ");
	if ((len = strlen (size)) > 10)
		WARNING ("gz_tar_list:", _("size field too long"))
	entry->size = g_strdup (size);
	/*    DEBUG ("gz_tar_list:", entry->size); */
    
	g_strchug (pes); /* Get Out spaces */
	date = strsep(&pes, " ");
	if ((len = strlen (date)) > 10)
		WARNING ("gz_tar_list:", _("date field too long"))
	entry->date = g_strdup (date);
	/*    DEBUG ("gz_tar_list:", entry->date); */
    
	g_strchug (pes); /* Get Out spaces */
	time = strsep(&pes, " ");
	if ((len = strlen (time)) > 8)
		WARNING ("gz_tar_list:", _("time field too long"))
	entry->time = g_strdup (time);
	/*    DEBUG ("gz_tar_list:", entry->time); */

	/* discard the linked filename and arrow */
	if (*perms == 'l') *(strstr (pes, " -> ")) = '\0';
	
	g_strstrip (pes); /* Get Out spaces */
	len = strlen (pes) -1;
	if (pes [len] == '/')
	{
		pes [len] = '\0';
		lastslash = '/';
	}
	path = pes;
	if ((name = strrchr (path, '/')))
	{
		name++;
		if ((len = (strlen (path) - strlen (name))) > PATH_MAX)
			WARNING ("gz_tar_list:", _("path name too long"))
		entry->path = g_malloc0 (len+1);
		strncpy (entry->path, path, len);
	}
	else
	{
		entry->path = g_strdup_printf ("");
		name = pes;
	}
	/*    DEBUG ("gz_tar_list:", entry->path); */	
	if ((len = strlen (name)) > NAME_MAX) 
		WARNING ("gz_tar_list:", _("name field too long"))
	if (len !=0 )
	{
		entry->name = g_strdup_printf ("%s%c", name, lastslash);
	}
	else
	{
		/* this is a directory */
		entry->name = g_strdup_printf ("%c", lastslash);
	}
/*	else entry->name = g_strdup_printf ("<DIR>"); */
	/*    DEBUG ("gz_tar_list:", entry->name); */
	
	return entry;
}
	
int
gz_tar_extract (gchar *filename, GList *extr_list, gchar *extr_dir,int compression)
{
  char *command;
  char *errstr;
  char topt [5];
  FILE *input = NULL;
  gchar *barwname = "appbar1";
  gint innum = 0;
  GtkWidget *mainappbar;
  gchar *tempfile;
  gchar *flopt;
  gchar *lfnam;
  int tempfd, num, i;
  gchar cr='\n';

  mainappbar = lookup_widget (main_win, barwname);
  
  gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,0);
  
  switch (compression)
  {
      case 0:
         sprintf (topt, "xvf");
         break;
      case 1:
         sprintf (topt, "xvzf");
         break;
      case 2:
         sprintf (topt, "xvIf");
         break;
      default:
         FATAL_ERROR ("gz_tar_extract:", _("Wrong Parameter"))
  }

  if (extr_list)
  {
      tempfile = g_strdup_printf ("%s/%s", config.dirs.scratch, TEMPFILE);
      if ((tempfd = mkstemp (tempfile)) == -1)
      {
         FATAL_ERROR ("gz_tar_add:", _("Unable to create temporary file"))
	 return -1;
      }
      DEBUG ("gz_tar_add: temp file:", tempfile);
			   
      num = g_list_length (extr_list);
      for (i = 0; i < num; i++)
      {
         lfnam = g_list_nth_data (extr_list, i);
	 write (tempfd, lfnam, strlen (lfnam));
	 write (tempfd, &cr, 1);
      }
      close (tempfd);

      flopt = g_strdup_printf ("-T %s", tempfile);
  }
  else
  {
      flopt = g_strdup_printf ("");
  }
							     
  errstr = g_strdup_printf ("Extract %s to: %s\n", filename, extr_dir);
  DEBUG ("gz_tar_extract:", errstr);
  command = g_strdup_printf ("%s %s \"%s\" -C \"%s\" %s", config.progs.tar, topt, filename, extr_dir, flopt);
  DEBUG ("tar command:", command);
  input = popen (command, "r");
  while (TRUE)
  {
      char entrystr [8192];

      fgets (entrystr, 8192, input);
      if (ferror (input)) FATAL_ERROR ("gz_tar_extract:",strerror (ferror (input)))
          /* Check if the line is not too long */
      if (feof (input)) break;
      DEBUG ("gz_tar_extract:", entrystr);
      /* update prograss bar */
      g_strchomp (entrystr);
      if (entrystr[strlen (entrystr) - 1] != '/')
      {
          gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) ++innum/flnum);
          while (gtk_events_pending ())
                 gtk_main_iteration ();
      }

  }
  pclose (input);
  return FALSE;
}

int 
gz_tar_add (gchar *filename, GList *add_list, gchar *add_dir, int computil)
{
  char *command = NULL;
  char *entrystr, *debugstr;
  FILE  *input = NULL;
  FILE  *output = NULL;
  gchar barwname[] = "appbar1";
  gchar *lfnam;
  gchar cr = '\n';
  char *tempfile = NULL;
  char *temptar = NULL;
  int tempfd, tarfd;
  int num, i, count;
  gint innum = 0;
  gint totnum = 0;
  GtkWidget *mainappbar;
  struct stat filestat;
  GtkWidget *main_clist = NULL;
  gz_list_entry *entry;

  main_clist = gz_mainclist_get_widget ();
  mainappbar = lookup_widget (main_win, barwname);
  gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,0);

  tempfile = g_strdup_printf ("%s/%s", config.dirs.scratch, TEMPFILE);
  if ((tempfd = mkstemp (tempfile)) == -1) 
  {
	FATAL_ERROR ("gz_tar_add:", _("Unable to create temporary file"))
	return -1;
  }
  DEBUG ("gz_tar_add: temp file:", tempfile);
  
  num = g_list_length (add_list);
  for (i = 0; i < num; i++)
  {
	lfnam = g_list_nth_data (add_list, i);
	write (tempfd, lfnam, strlen (lfnam));
	write (tempfd, &cr, 1);
  }
  close (tempfd);
 
  /* just to let the bar move if we have only a dirname */
  if (num < 2) num = 10;
  
  switch (computil)
  {
      case 0:
	temptar = g_strdup_printf ("\"%s\"", filename);
	totnum = num;
	break;
      case 1:
	command = g_strdup_printf ("%s -c \"%s\"", config.progs.gunzip, filename);
	break;
      case 2:
	command = g_strdup_printf ("%s -c \"%s\"", config.progs.bunzip2, filename);
	break;
      default:
         FATAL_ERROR ("gz_tar_add:", _("Wrong Compression Parameter"))
  }
  if (computil)
  {
	temptar = g_strdup_printf ("%s/%s", config.dirs.scratch, TEMPFILE);
	if ((tarfd = mkstemp (temptar)) == -1) 
	{
		FATAL_ERROR ("gz_tar_add:", _("Unable to create temporary file"))
		return -1;
	}
	DEBUG ("gz_tar_add: temp tar:", temptar);

	DEBUG ("tar temp command:", command);
	input = popen (command, "r");
	entrystr = g_malloc (TAR_ZBUFFER);
	while (!feof (input))
	{
		count = fread (entrystr, 1, TAR_ZBUFFER, input);
		if (ferror (input)) FATAL_ERROR ("gz_tar_add:",strerror (ferror (input)))
		/* while (gtk_events_pending ()) gtk_main_iteration (); */ /* need it ?*/
		write (tarfd, entrystr, count);
	}
	totnum = num +1;
	gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) ++innum/totnum);
	g_free (entrystr);
	g_free (command);
	pclose (input);
	close (tarfd);
  }
  DEBUG ("gz_tar_add: Adding files to", temptar);
  command = g_strdup_printf ("%s rvvf %s -C \"%s\" -P -T %s", config.progs.tar, temptar, add_dir,  tempfile);
  DEBUG ("tar add command:", command);
  input = popen (command, "r");
  entrystr = g_malloc (8192);
  while (TRUE)
  {
      fgets (entrystr, 8192, input);
      if (ferror (input)) FATAL_ERROR ("gz_tar_add:",strerror (ferror (input)))
          /* Check if the line is not too long */
      if (feof (input)) break;
      /* Read file names and put them in the main clist */
/*      g_strchomp (entrystr);
      entry = gz_tar_entry (entrystr);
      gtk_clist_append ((GtkCList *)main_clist, (gpointer)entry); */
      DEBUG ("gz_tar_add:", entrystr);
      /* update prograss bar */
      g_strchomp (entrystr);
      if (entrystr[strlen (entrystr) - 1] != '/')
      {
	  if (innum == totnum) innum = 0; /* I can't know how many files there are */
          gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) ++innum/totnum);
          while (gtk_events_pending ()) gtk_main_iteration ();
      }

  }
  gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) 1); /* fill bar */
  g_free (entrystr);
  g_free (command);
  pclose (input);
/*  unlink (tempfile); */
  g_free (tempfile);
  switch (computil)
  {
      case 0:
	break;
      case 1:
	command = g_strdup_printf ("%s -c > \"%s\"", config.progs.gzip, filename);
	break;
      case 2:
	command = g_strdup_printf ("%s -c > \"%s\"", config.progs.bzip2, filename);
	break;
      default:
	/* should never happen */
        FATAL_ERROR ("gz_tar_add:", _("Internal Error - Wrong Compression Parameter(2)"))
  }
  if (computil)
  {
	DEBUG ("gzip file", temptar);
	DEBUG ("tar untemp command:", command);
	if (!(output = popen (command, "w")))
	{
		ERROR ("gz_tar_add", _("Unable to launch gzip"))
		return -1;
	}
	stat (temptar, &filestat);
	totnum = filestat.st_size;
#ifdef GZ_DEBUG
	debugstr = g_strdup_printf ("Total file size %d\n", totnum);
	DEBUG ("gz_tar_add", debugstr);
	g_free (debugstr);
#endif /* GZ_DEBUG */	innum = 0;
	if ((input = fopen (temptar, "r")) == NULL)
	{
		ERROR ("gz_tar_add", strerror (ferror (output)))
		return -1;
	}	
	entrystr = g_malloc (TAR_ZBUFFER);
	while (!feof (input))
	{
		count = fread (entrystr, 1, TAR_ZBUFFER, input);
		if (ferror (input))
			FATAL_ERROR ("gz_tar_add:", strerror (errno))
		innum += count;
		fwrite (entrystr, 1, count, output);
		if (ferror (output))
			FATAL_ERROR ("gz_tar_add:", strerror (ferror (output)))
		gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat)innum/totnum);
		while (gtk_events_pending ()) gtk_main_iteration ();
	}
	g_free (entrystr);
	g_free (command);
	pclose (output);
	fclose (input);
	unlink (temptar);
  }
  g_free (temptar);

  /* quickfix to see icons */
  gz_open (filename);
  return 0;
}

int 
gz_tar_delete (gchar *filename, GList *del_list, int computil)
{
  char *command = NULL;
  char *entrystr, *debugstr;
  FILE  *input = NULL;
  FILE  *output = NULL;
  gchar barwname[] = "appbar1";
  gchar *lfnam;
  gchar cr = '\n';
  char *tempfile = NULL;
  char *temptar = NULL;
  int tempfd, tarfd;
  int num, i, count;
  gint innum = 0;
  gint totnum = 0;
  GtkWidget *mainappbar;
  struct stat filestat;
  GtkWidget *main_clist = NULL;
  gz_list_entry *entry;

  main_clist = gz_mainclist_get_widget ();
  mainappbar = lookup_widget (main_win, barwname);
  gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,0);

  tempfile = g_strdup_printf ("%s/%s", config.dirs.scratch, TEMPFILE);
  if ((tempfd = mkstemp (tempfile)) == -1) 
  {
	FATAL_ERROR ("gz_tar_del:", _("Unable to create temporary file"))
	return -1;
  }
  DEBUG ("gz_tar_del: temp file:", tempfile);
  
  num = g_list_length (del_list);
  for (i = 0; i < num; i++)
  {
	lfnam = g_list_nth_data (del_list, i);
	write (tempfd, lfnam, strlen (lfnam));
	write (tempfd, &cr, 1);
  }
  close (tempfd);
  
  switch (computil)
  {
      case 0:
	temptar = g_strdup_printf ("\"%s\"", filename);
	totnum = num;
	break;
      case 1:
	command = g_strdup_printf ("%s -c \"%s\"", config.progs.gunzip, filename);
	break;
      case 2:
	command = g_strdup_printf ("%s -c \"%s\"", config.progs.bunzip2, filename);
	break;
      default:
         FATAL_ERROR ("gz_tar_add:", _("Wrong Compression Parameter"))
  }
  if (computil)
  {
	temptar = g_strdup_printf ("%s/%s", config.dirs.scratch, TEMPFILE);
	if ((tarfd = mkstemp (temptar)) == -1) 
	{
		FATAL_ERROR ("gz_tar_del:", _("Unable to create temporary file"))
		return -1;
	}
	DEBUG ("gz_tar_del: temp tar:", temptar);

	DEBUG ("tar temp command:", command);
	input = popen (command, "r");
	entrystr = g_malloc (TAR_ZBUFFER);
	while (!feof (input))
	{
		count = fread (entrystr, 1, TAR_ZBUFFER, input);
		if (ferror (input)) FATAL_ERROR ("gz_tar_del:",strerror (ferror (input)))
		/* while (gtk_events_pending ()) gtk_main_iteration (); */ /* need it ?*/
		write (tarfd, entrystr, count);
	}
	totnum = num +1;
	gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) ++innum/totnum);
	g_free (entrystr);
	g_free (command);
	pclose (input);
	close (tarfd);
  }
  DEBUG ("gz_tar_del: Deleting files from", temptar);
  command = g_strdup_printf ("%s --delete -f %s -T %s", config.progs.tar, temptar, tempfile);
  DEBUG ("tar del command:", command);
  /* Execute command */
  system (command);

  g_free (command);
  unlink (tempfile);
  g_free (tempfile);
  switch (computil)
  {
      case 0:
	break;
      case 1:
	command = g_strdup_printf ("%s -c > \"%s\"", config.progs.gzip, filename);
	break;
      case 2:
	command = g_strdup_printf ("%s -c > \"%s\"", config.progs.bzip2, filename);
	break;
      default:
	/* should never happen */
        FATAL_ERROR ("gz_tar_del:", _("Internal Error - Wrong Compression Parameter(2)"))
  }
  if (computil)
  {
	DEBUG ("gzip file", temptar);
	DEBUG ("tar untemp command:", command);
	if (!(output = popen (command, "w")))
	{
		ERROR ("gz_tar_del", _("Unable to launch gzip"))
		return -1;
	}
	stat (temptar, &filestat);
	totnum = filestat.st_size;
#ifdef GZ_DEBUG
	debugstr = g_strdup_printf ("Total file size %d\n", totnum);
	DEBUG ("gz_tar_del", debugstr);
	g_free (debugstr);
#endif /* GZ_DEBUG */
	innum = 0;
	if ((input = fopen (temptar, "r")) == NULL)
	{
		ERROR ("gz_tar_del", strerror (ferror (output)))
		return -1;
	}	
	entrystr = g_malloc (TAR_ZBUFFER);
	while (!feof (input))
	{
		count = fread (entrystr, 1, TAR_ZBUFFER, input);
		if (ferror (input))
			FATAL_ERROR ("gz_tar_del:", strerror (errno))
		innum += count;
		fwrite (entrystr, 1, count, output);
		if (ferror (output))
			FATAL_ERROR ("gz_tar_del:", strerror (ferror (output)))
		gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat)innum/totnum);
		while (gtk_events_pending ()) gtk_main_iteration ();
	}
	/* Be sure the bar is full when operations finish */
	gnome_appbar_set_progress (GNOME_APPBAR (mainappbar), 1);
	while (gtk_events_pending ()) gtk_main_iteration ();
	
	g_free (entrystr);
	g_free (command);
	pclose (output);
	fclose (input);
	unlink (temptar);
  }
  g_free (temptar);

  /* reopen the file to show what remain inside */
  gz_open (filename);
  
  return 0;
}
