/*
 * zip.c
 * This file provides the zip file operations interface
 */

#include "common.h"
#include "zip.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

extern gz_config config;

GList *
gz_zip_list (const gchar *filename)
{
  GList *list = NULL;
  char  *command;
  FILE  *input = NULL;
  gz_list_entry *entry;

  command = g_strdup_printf ("%s -l -qq \"%s\"", config.progs.unzip, filename);
  DEBUG ("zip command:", command)
  input = popen (command, "r");
  if (input == NULL) ERROR ("gz_zip_list", _("cannot execute zip command"))

  while (TRUE)
  {
    char *size, *date, *time, *name, *path;
    char entrystr [255 +PATH_MAX +NAME_MAX]; /* No more than 255 size/date/... chars */
    char *pes; /* entry string pointer */
    int len; /* string lenght */
    char lastslash = '\0';

    /* Read a line at a time */
    entrystr[0] = '\0';
    fgets (entrystr, 255 +PATH_MAX +NAME_MAX, input);
    if (ferror (input)) FATAL_ERROR ("gz_zip_list:",strerror (ferror (input)))
        /* Check if the line is not too long */
    if (feof (input)) break;
    if (entrystr[strlen (entrystr) - 1] != '\n')
        WARNING ("gz_zip_list:", _("Too many chars in a file entry"))
    entrystr[strlen (entrystr) - 1] = '\0';
    DEBUG ("gz_zip_list:", entrystr)

    g_strchomp (entrystr);

    if (entrystr[strlen (entrystr) - 1] != '/') /* do not show stored path as files */
    {

        entry = g_malloc0 (sizeof(gz_list_entry));

        /* entry->perms = NULL; info not available */
        /* entry->owner = NULL; info not available */

        pes = entrystr;

        g_strchug (pes); /* Get Out spaces */
        size = strsep(&pes, " ");
        if ((len = strlen (size)) > 10)
          WARNING ("gz_zip_list:", _("size field too long"))
        entry->size = g_strdup (size);
	/*    DEBUG ("gz_zip_list:", entry->size); */
     
        g_strchug (pes); /* Get Out spaces */
        date = strsep(&pes, " ");
        if ((len = strlen (date)) > 8)
          WARNING ("gz_zip_list:", _("date field too long"))
        entry->date = g_strdup (date);
	/*    DEBUG ("gz_zip_list:", entry->date); */
	 
        g_strchug (pes); /* Get Out spaces */
        time = strsep(&pes, " ");
        if ((len = strlen (time)) > 5)
          WARNING ("gz_zip_list:", _("time field too long"))
        entry->time = g_strdup (time);
	/*    DEBUG ("gz_zip_list:", entry->time); */
    
        g_strchug (pes); /* Get Out spaces */
	path = pes;
	if (path [strlen (path)] == '/')
	{
		path [strlen (path)] == '\0';
		lastslash = '/';
	}
	if ((name = strrchr (path, '/')))
	{
	    name++;
	    
	    if ((len = (strlen (path) - strlen (name))) > PATH_MAX)
		 WARNING ("gz_zip_list:", _("path name too long"))
	    entry->path = g_malloc0 (len+1);
	    strncpy (entry->path, path, len);
	/*    DEBUG ("gz_zip_list:", entry->path); */
	}
	else name = pes;
	if ((len = strlen (name)) > NAME_MAX) 
          WARNING ("gz_zip_list:", _("name field too long"))
	if (len !=0 )
	{
		entry->name = g_strdup_printf ("%s%c", name, lastslash);
	}
	else
	{
		/* this is a directory */
		entry->name = g_strdup_printf ("%c", lastslash);
	}
/*	else entry->name = g_strdup_printf ("<DIR>"); */
	/*    DEBUG ("gz_zip_list:", entry->name); */

        list = g_list_append (list, entry);
    
    }
    
  }

  pclose (input);
  g_free (command);

  return list;

}

int
gz_zip_extract (gchar *filename, gchar *extr_dir)
{
  char *command;
  char *errstr;
  FILE  *input = NULL;
  gchar *barwname = "appbar1";
  gint innum = -1; /* unzip put an extra garbage line on stdout */
  GtkWidget *mainappbar;
  
  mainappbar = lookup_widget (main_win, barwname);

  errstr = g_strdup_printf ("Extract %s to: %s\n", filename, extr_dir);
  DEBUG ("gz_zip_extract:", errstr);
  g_free (errstr);
  command = g_strdup_printf ("%s -o \"%s\" -d \"%s\"", config.progs.unzip, filename, extr_dir);
  DEBUG ("zip command:", command);
  input = popen (command, "r");
  while (TRUE)
  {
    char entrystr [8192];

    fgets (entrystr, 8192, input);
    if (ferror (input)) FATAL_ERROR ("gz_zip_extract:",strerror (ferror (input)))
        /* Check if the line is not too long */
    if (feof (input)) break;
    DEBUG ("gz_zip_extract:", entrystr);
    /* update prograss bar */
    gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) ++innum/flnum);
    while (gtk_events_pending ())
                gtk_main_iteration ();

  }
  pclose (input);
  return FALSE;
}

int
gz_zip_add (gchar *filename, GList *add_list, gchar *add_dir, int complevel)
{
  char *command = NULL;
  char *entrystr;
  FILE  *input = NULL;
  FILE  *output = NULL;
  gchar barwname[] = "appbar1";
  gchar *lfnam;
  gchar cr = '\n';
  char *tempfile = NULL;
  int tempfd, tarfd;
  int num, i, count;
  int totnum, innum;
  GtkWidget *mainappbar;
  struct stat filestat;
  GtkWidget *main_clist = NULL;
  gz_list_entry *entry;

  main_clist = gz_mainclist_get_widget ();
  mainappbar = lookup_widget (main_win, barwname);
  gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,0);

  tempfile = g_strdup_printf ("%s/%s", config.dirs.scratch, TEMPFILE);
  if ((tempfd = mkstemp (tempfile)) == -1) 
  {
	FATAL_ERROR ("gz_zip_add:", _("Unable to create temporary file"))
	return -1;
  }
  DEBUG ("gz_zip_add: temp file:", tempfile);
  
  totnum = num = g_list_length (add_list);
  for (i = 0; i < num; i++)
  {
	lfnam = g_list_nth_data (add_list, i);
	write (tempfd, lfnam, strlen (lfnam));
	write (tempfd, &cr, 1);
  }
  close (tempfd);
  
  DEBUG ("gz_zip_add: Adding files to", filename);
  command = g_strdup_printf ("cd \"%s\" && %s -r -%d -@ \"%s\"", add_dir, config.progs.zip, complevel, filename);
  DEBUG ("zip add command:", command);
  output = popen (command, "w");
  entrystr = g_malloc (8192);
  input = fopen (tempfile, "r");
  
  innum = 0;
  while (!feof (input))
  {
      fgets (entrystr, 8192, input);
       DEBUG ("gz_zip_add: temp file line = ", entrystr);
      if (ferror (input)) FATAL_ERROR ("gz_zip_add:",strerror (ferror (input))) break;
      fwrite (entrystr, 1, 8192, output);
      if (ferror (output)) FATAL_ERROR ("gz_zip_add:",strerror (ferror (input))) break;
      gnome_appbar_set_progress (GNOME_APPBAR (mainappbar) ,(gfloat) ++innum/totnum);
  }
  pclose (output);
  fclose (input);
  g_free (entrystr);
  g_free (command);
 /*  unlink (tempfile); */
  g_free (tempfile);
  return 0;
}
